@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_service - remote control daemon for Nokia N900 phone#
-# $Id: smscon_service.py 168 2012-05-12 19:04:44Z yablacky $
+# $Id: smscon_service.py 180 2012-05-22 23:42:47Z yablacky $
 ##############################################################
 
 # standard modules
@@ -41,6 +41,7 @@ PingCountMax                = 3                                 # max number of
 LastKnownGoodConfigFile     = '.%s_lkg' % smscon.NAME
 GpsModeLOCATION             = 'location'
 GpsModeTRACKING             = 'tracking'
+AmbientLightSensorDevice    = '/sys/class/i2c-adapter/i2c-2/2-0029/lux'
 
 #############################################
 #    variables
@@ -526,7 +527,8 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
     # Check if Command is a specific valid SMSCON command:
     if not (Command in [smscnf[cmd] for cmd in smscnf.getSmsCommandList() if smscnf[cmd] != '']
             or smscnf.parseSmsCommand(Command, smscnf.COM_CUSTOM)
-            or smscnf.parseSmsCommand(Command, smscnf.COM_SHELL)):
+            or smscnf.parseSmsCommand(Command, smscnf.COM_SHELL)
+            or smscnf.parseSmsCommand(Command, smscnf.COM_CAMERA)):
         if LogIfCmdUnknown:
             LOGGER.error( '"%s" is not a valid %s command.' % (Command, smscon.NAME) )
             LogBatteryChargeInfo()
@@ -613,13 +615,16 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
         else:
             DisableSSHcontrol()
             
-    elif Command == smscnf.COM_CAMERA:
+    elif smscnf.parseSmsCommand(Command, smscnf.COM_CAMERA, CommandArgs):
+        os.putenv('SMSCON_COMMANDORIGIN', CommandOrigin)
+        os.putenv('SMSCON_SENDERNUMBER', smscnf.SENDERNUMBER)
+        os.putenv('SMSCON_MASTERNUMBER', smscnf.MASTERNUMBER)
         behaviourChanged = False
         if not IsInternetReachable():
             SetNetworkConnectionType('GPRS') # enable GPRS data network
             behaviourChanged = SetAutoConnectBehaviour('GPRS') # disable switching to other network
 
-        PerformCamera()
+        PerformCamera(CommandArgs['args'])
 
         if behaviourChanged:
             SetAutoConnectBehaviour(ConnectionBehaviour)
@@ -1014,15 +1019,45 @@ def PerformPoweroff():
 #   camera
 #############################################
 
-def PerformCamera():
+def PerformCamera(args = []):
     """
     Get picture of front camera and send it to email address.
     """
+    LOGGER.debug('PerformCamera(' + ','.join(args) + ')')
+    
+    # Should there be a running live stream, stop it in any case.
+    # Otherwise neither a new stream could be started nor a picture may be taken. 
+    LiveCameraCmd = os.path.join(InstPath, 'live-camera.sh')
+    if os.path.isfile(LiveCameraCmd):
+        PIDs = pidof(LiveCameraCmd)
+        LOGGER.debug('%s PIDS: (%s)' % (LiveCameraCmd, PIDs))
+        if PIDs:
+            os.system('kill -TERM %s' % PIDs)
+            LOGGER.info(         'Camera live stream stopped.')
+            RevealThat('Camera', 'Camera live stream stopped.')                    
+            time.sleep(2)
+    
+        if not (len(args) < 2 or args[0].upper() in ('', 'P', 'PIC', 'IMG')):
+            # Interpret this as command to control camera live stream.
+            if args[0] and not (args[0].upper() in ('off', '0', '-')):
+                if os.WEXITSTATUS(os.system('"%s" 1 %s &' %(LiveCameraCmd, ' '.join(shellArg(x) for x in args[1:])))) == 0:
+                    LiveViewerCmd = os.path.join(InstPath, 'live-viewer.sh')
+                    LOGGER.info(         'Camera live stream started.')
+                    RevealThat('Camera',['Camera live stream started (%s).\n' % ','.join(args),
+                                         'Viewer is attached. Call with --help for details.'],
+                               Attachment = os.path.isfile(LiveViewerCmd) and LiveViewerCmd,
+                               Attachhint = 'text/plain')
+                else:
+                    LOGGER.error(        'Camera live stream did not start.')
+                    RevealThat('Camera', 'Camera live stream did not start.')
+            return
+
+    # Take a single image:
     ImageFilename = TakeFrontCamPicture()
     if ImageFilename:
         Hostname = os.uname()[1]
         EMAILsend(EmailTo = smscnf.EMAILADDRESS,
-                  Subject =  'From "%s" command / N900[%s]' % (smscnf.COM_CAMERA, Hostname),
+                  Subject =  'NOKIA N900[%s]: %s' % (Hostname, smscnf.COM_CAMERA),
                   Text    =  ['Frontcam picture from Nokia N900 phone:\n',
                               'Taken on %s at %s\n' % (Hostname, GetTimeString())],
                   SuccessLogMsg  = '(EMAIL) camera picture send to "%s"' % smscnf.EMAILADDRESS,
@@ -1037,15 +1072,31 @@ def TakeFrontCamPicture():
     ImageFilename       = os.path.join(LogPath, PhotoName)
     ImageFileEncoding   = 'jpegenc'     # TODO: encoding should depend on desired image type (png, jpeg, ...) 
     CamDevice           = '/dev/video1'
-    DelaySeconds        = 1
+    
+    # If the camctrl tool is available, then before taking picture, let it measure
+    # ambient light and set appropriate gain and exposure values for the camera: 
+    CamCtrl = os.path.join(InstPath, 'camctrl')
+    if os.path.isfile(CamCtrl):
+        try:    os.WEXITSTATUS(os.system(CamCtrl
+                        + " --device '%s'" % CamDevice
+                        + " --ambient-light-sensor '%s'" % AmbientLightSensorDevice
+                        + " --all-auto-via-als 1"))
+        except: pass
+
     GStreamerPipeline   = (
+        # For an acceptable image let the camera warm up for approx 0.5 seconds.
+        # This is achieved by taking 16 buffers (rather than only one) and then
+        # use the last one for the image.
         "v4l2camsrc device='%s' num-buffers=16" % CamDevice,
+#        # framerate-cap requires videorate element:
+#       "videorate",
+#       "video/x-raw-yuv,width=640,height=480,framerate=3/1",
         "video/x-raw-yuv,width=640,height=480",
         "ffmpegcolorspace",
-        "gamma gamma=1.8",
-        "videobalance hue=0.1 brightness=0.2 saturation=1 contrast=1",
+        "gamma gamma=2",
         ImageFileEncoding,
-        "filesink location='%s'" % ImageFilename,
+        # Let multifilesink store all images to the same file; so the last one wins:
+        "multifilesink render-delay=1000000000 location='%s'" % ImageFilename,
     )
 
     try:    os.remove(ImageFilename)
@@ -1055,8 +1106,6 @@ def TakeFrontCamPicture():
         LOGGER.error('Camera picture failed (image capture error).')
         return False
 
-    time.sleep(DelaySeconds)        # i guess this this not really required... but to be on save side...
-
     if not os.path.isfile(ImageFilename):
         LOGGER.error('Taking camera picture failed (image file not found).')
         return False
@@ -1747,15 +1796,19 @@ def ValidateIMSI():
     CurrentIMSI = smscon.GetPhoneCode('IMSI')
 
     if CurrentIMSI == None:
-        LOGGER.info('No SIM card present.')
-        ExitDaemon()
+        LOGGER.warning( 'SIM card not present (or just inactive if user skipped or failed to enter SIM pin).')
+        # With some luck internet connection via wlan is available.
+        SendMethod = 'email'    # sms would not work w/o a SIM...
+        RevealThat('No or skipped SIM',
+                        'SIM card not present (or just inactive if user skipped or failed to enter SIM pin).',
+                        SendMethod)
 
     # compare current IMSI code with saved one or more in code file.
     elif CurrentIMSI in ReadIMSI():
-        LOGGER.info('Authorized IMSI code found.')
+        LOGGER.info('SIM card with authorized IMSI code found.')
 
     else:
-        LOGGER.warning('IMSI code has changed to "%s".' % CurrentIMSI)
+        LOGGER.warning('SIM card not yet autorized, IMSI code has changed to "%s".' % CurrentIMSI)
 
         # save new IMSI code to file
         WriteIMSI(CurrentIMSI)    
@@ -1934,7 +1987,7 @@ def GetAcceleratorValues():
 
 def GetProximityState():
     """
-    Get current ambient light value.
+    Get current proximity value.
     @return string. 1 open, closed or unknown
     """
     return SaveReadFileLine('/sys/devices/platform/gpio-switch/proximity/state',
@@ -1945,7 +1998,7 @@ def GetAmbientLightValue():
     Get current ambient light value.
     @return string. 1 integer [lux] or unknown
     """
-    return SaveReadFileLine('/sys/class/i2c-adapter/i2c-2/2-0029/lux',
+    return SaveReadFileLine(AmbientLightSensorDevice,
                 ErrorMessage = 'Failed to read ambient light sensor')
 
 #############################################
