@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon - remote control library for Nokia N900 phone       #
-# $Id: smsconlib.py 168 2012-05-12 19:04:44Z yablacky $
+# $Id: smsconlib.py 171 2012-05-21 00:47:26Z yablacky $
 ##############################################################
 
 # standard modules
@@ -34,7 +34,7 @@ NAME = 'smscon'
 VERSION = '999.1'   # for trunk and private builds etc.
 
 NAME, VERSION = getSysBuildNameAndVersion(NAME, VERSION,
-    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.18/src/opt/smscon/smsconlib.py 168 2012-05-12 19:04:44Z yablacky $',
+    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.19/src/opt/smscon/smsconlib.py 171 2012-05-21 00:47:26Z yablacky $',
     dirLevels   = 3 )
     
 XDEBUG   = False        # cross debug (use only if not running on phone)
@@ -135,7 +135,7 @@ def shellArg(s):
 def pidof(ProcessName):
     """
     Determine the PID of processes that match a given name.
-    @return string Whitespace separator PID. Empty if none.
+    @return string Whitespace separated PID(s). Empty if none.
     """
     PIDs = os.popen('pidof %s' % shellArg(ProcessName)).read().strip()
     if PIDs != '':
